@@ -6,29 +6,30 @@ package UR::Namespace::Command::Define::Datasource;
 use warnings;
 use strict;
 use UR;
-our $VERSION = "0.46"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
-    is => "UR::Namespace::Command::Base",
-    has_optional => [
+    is => "UR::Namespace::Command",
+    has => [
         dsid => {
-            is => 'Text',
-            doc => "The full class name to give this data source.",
+            type => 'String',
+            doc => "The class name to give this data source. You must supply dsid or dsname",
+            is_optional => 1,
         },
         dsname => {
-            is => 'Text',
-            shell_args_position => 1,
-            doc => "The distinctive part of the class name for this data source.  Will be prefixed with the namespace then '::DataSource::'.",
+            type => 'String',
+            doc => "Basic name to give this data source.  The class name will become namespace::DataSource::dsname",
+            is_optional => 1,
        },
     ],
-    doc => 'add a data source to the current namespace'
 );
 
-sub _is_hidden_in_docs { 1 }
-
 sub sub_command_sort_position { 2 }
 
+sub help_brief {
+   "Add a data source to the current namespace.";
+}
+
 sub data_source_module_pathname {
     my $self = shift;
 
