@@ -2,11 +2,8 @@ package UR::Namespace::Command::List::Objects;
 
 use strict;
 use warnings;
-require UR;
-our $VERSION = "0.44"; # UR $VERSION;
 
 use above "UR";
-use UR::Object::Command::List;
 
 class UR::Namespace::Command::List::Objects { 
   is =>  'UR::Object::Command::List',
@@ -14,5 +11,5 @@ class UR::Namespace::Command::List::Objects {
 
 1;
 
-#$HeadURL: svn+ssh://svn/srv/svn/gscpan/distro/ur-bundle/trunk/lib/UR/Namespace/Command/List/Objects.pm $
+#$HeadURL: svn+ssh://svn/srv/svn/gscpan/distro/ur-bundle/releases/UR-0.5/lib/UR/Namespace/Command/List/Objects.pm $
 #$Id: Objects.pm 36327 2008-07-08 20:59:29Z ebelter $
