@@ -18,10 +18,8 @@ UR::Object::Type->define(
     ],
 );
 
-sub sub_command_sort_position { 2 };
-
 sub help_brief {
-    "Update class definitions (and data dictionary cache) to reflect changes in the database schema."
+    "Update table definitions and class definitions to reflect changes in the remote data dictionary."
 }
 
 sub help_detail {
@@ -96,6 +94,12 @@ $DB::single=1;
     #    }
     #}
 
+    if (@{ $self->bare_args }) {
+        $self->error_message("Bare paramters not supported: @{ $self->bare_args }\n");
+        $self->status_message($self->help_usage_complete_text,"\n");
+        return;
+    }
+
     $self->_init;
 
     my $namespace = $self->namespace_name;
@@ -341,7 +345,6 @@ $DB::single=1;
             @changed_class_meta_objects
         ) {
             my $class_name = $obj->class_name;
-            next unless $class_name;  #if $obj is a ghost, class_name might return undef?
             $changed_classes{$class_name} = 1;
         }
         unless (@changed_class_meta_objects) {
@@ -425,6 +428,7 @@ $DB::single=1;
 #   ->_sync_filesystem()
 #
 
+
 sub _update_database_metadata_objects_for_schema_changes {
     my ($self, %params) = @_;
     my $data_source = delete $params{data_source};
@@ -447,7 +451,7 @@ sub _update_database_metadata_objects_for_schema_changes {
 
     # from the database now
     my @current_table_names = $data_source->_get_table_names_from_data_dictionary();
-    my %current_table_names = map { s/"|'//g; uc($_) => $_ } @current_table_names;
+    my %current_table_names = map { s/"|'//g; uc($_) => 1 } @current_table_names;
 
     my %all_table_names = (%current_table_names, %previous_table_names);
 
@@ -464,10 +468,6 @@ sub _update_database_metadata_objects_for_schema_changes {
         my $last_recorded_ddl_time;
         my $last_object_revision;
 
-        # UR always keeps table names stored in upper-case.  Some databases (mysql)
-        # are case sensitive when querying the data dictionary
-        my $db_table_name = $current_table_names{$table_name};
-
         eval {
             #($table_object) = $data_source->get_tables(table_name => $table_name);
 
@@ -486,7 +486,7 @@ sub _update_database_metadata_objects_for_schema_changes {
                     $dsn . " " . $table_name
                 )
             );
-            my $table_object = $data_source->refresh_database_metadata_for_table_name($db_table_name);
+            my $table_object = $self->_update_database_metadata_objects_for_table_changes($data_source,$table_name);
             next unless $table_object; 
 
             $table_object->last_ddl_time($last_ddl_time_for_table_name->{$table_name});
@@ -500,13 +500,14 @@ sub _update_database_metadata_objects_for_schema_changes {
             ) {
                 my $last_update = $table_object->last_ddl_time || $table_object->last_object_revision;
                 my $this_update = $last_ddl_time_for_table_name->{$table_name} || "<unknown date>";
-                my $table_object = $data_source->refresh_database_metadata_for_table_name($db_table_name);
+                #$table_object->delete;
+                my $table_object = $self->_update_database_metadata_objects_for_table_changes($data_source,$table_name);
                 unless ($table_object) {
                     #$DB::single = 1;
                     print;
                 }
                 my @changes =
-                #    grep { not  ($_->properties == 1 and ($_->properties)[0] eq "last_object_revision") }
+                    grep { not  ($_->properties == 1 and ($_->properties)[0] eq "last_object_revision") }
                     $table_object->__changes__;
                 if (@changes) {
                     $self->status_message(
@@ -712,14 +713,6 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             next;
         }
         $reference->constraint_name(undef);
-
-        my $property = $reference->property_meta();
-        unless ($property) {
-            $self->status_message(sprintf("~ No property found for deleted foreign key constraint %-32s %-32s class $class_name property %s\n",
-                                          $table->table_name, $fk->id, $reference->delegation_name));
-        }
-        $property->delete;
- 
     }
 
     # DELETED UNIQUE CONSTRAINTS
@@ -731,7 +724,6 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
     #  to function as it did before.
 
     # DELETED COLUMNS
-    my @saved_removed_column_messages;  # Delete them now, but report about them later in the 'Updating class properties' section
     for my $column (sort $sorter @{ $dd_changes_by_class{"UR::DataSource::RDBMS::TableColumn::Ghost"} }) {
         my $table = $column->get_table;
         unless ($table) {
@@ -763,21 +755,10 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             class_name => $class_name,
             property_name => $property->property_name,
         );
-        #my @reference_property_from = grep { $_->property_name eq $property->property_name }
-        #                              map { $_->reference_property_metas }
-        #                              UR::Object::Reference->get(class_name => $class_name);
-        #my @reference_property_to = UR::Object::Reference::Property->get(
-        #    r_class_name => $class_name,
-        #    r_property_name => $property->property_name,
-        #);
-        # FIXME -  Reference Properties are autovivified, and for that to work you must either 
-        # you must either use tha_id or both class_name and property_name.  We can work around this
-        # by figuring it out ourselves, but it'd be nice if UR::Object::Reference::Property->get()
-        # did it
-        my @reference_property_to   = grep { $_->r_property_name eq $property->property_name }
-                                      map { $_->reference_property_metas }
-                                      UR::Object::Reference->get(r_class_name => $class_name);
-
+        my @reference_property_to = UR::Object::Reference::Property->get(
+            r_class_name => $class_name,
+            r_property_name => $property->property_name,
+        );
         my @reference_ids = map { $_->reference_id } (@reference_property_from, @reference_property_to);
         for my $reference_property (@reference_property_from, @reference_property_to) {
             $reference_property->delete;
@@ -794,12 +775,12 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         }
 
         unless ($table->isa("UR::DataSource::RDBMS::Table::Ghost")) {
-            push(@saved_removed_column_messages, 
-                sprintf("D %-40s property %-16s for removed column %s.%s\n",
-                        $class->class_name,
-                        $property->property_name,
-                        $column->table_name, 
-                        $column->column_name,
+            $self->status_message(
+                sprintf(
+                    "D %-32s deleted from class %-40s for deleted column %-32s" . "\n",
+                    $property->property_name,
+                    $class->class_name,
+                    $column->column_name
                 )
             );
         }
@@ -817,7 +798,7 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
         # Though we create classes for tables, we don't immediately delete them, just deflate them.
         my $table_name = $table->table_name;
         unless ($table_name) {
-            $self->status_message("~ No table_name for deleted table object ".$table->id);
+            $self->status_message("~ No table_name for delete table object ".$table->id);
             next;
         }
 
@@ -854,7 +835,7 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             } sort keys %ancestory;
         if (@ancestors_with_tables) {
             $self->status_message(
-                sprintf("U %-40s class is now detached from deleted table %-32s.  It still inherits from classes with persistent storage." . "\n",$class_name,$table_name)
+                sprintf("U %-40s is now detached from deleted table %-32s.  It still inherits from classes with persistent storage." . "\n",$class_name,$table_name)
             );
         }
         else {
@@ -869,7 +850,8 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             $class->delete;
             #$DB::single = 1;
             $self->status_message(
-                sprintf("D %-40s class deleted for deleted table %s" . "\n",$class_name,$table_name)
+                #sprintf("D %-40s deleted for deleted table %-32s" . "\n",$class_name,$table_name)
+                sprintf("D %-40s deleted for deleted table %s" . "\n",$class_name,$table_name)
             );
         }
     } # next deleted table
@@ -924,11 +906,11 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             
             if ($class->__changes__) {
                 $self->status_message(
-                    sprintf("U %-40s class uses %s %s %s" . "\n",
-                            $class_name,
-                            $table->data_source->get_name,
-                            lc($table->table_type),
-                            $table_name)
+                    #sprintf("U %-40s uses table %-40s" . "\n",$class_name,$table_name)
+                    sprintf("U %-40s uses %s %s %s" . "\n",$class_name,
+                                                           $table->data_source->get_name,
+                                                           lc($table->table_type),
+                                                           $table_name)
                 );
             }
         }
@@ -957,12 +939,12 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             }
 
             $self->status_message(
-                     sprintf("A %-40s class uses %s %s %s" . "\n",
-                             $class_name,
-                             $table->data_source->get_name,
-                             lc($table->table_type),
-                             $table_name)
-                     );
+                     #sprintf("A %-40s uses table %-40s" . "\n",$class_name,$table_name)
+                     sprintf("A %-40s uses %s %s %s" . "\n",$class_name,
+                                                            $table->data_source->get_name,
+                                                            lc($table->table_type),
+                                                            $table_name)
+                  );
 
             my $type_name = $data_source->resolve_type_name_for_table_name($table_name);
             $type_name .= ' view' if ($table->table_type =~ m/view/i);
@@ -1001,8 +983,10 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
 
             unless ($class->class_name->isa('UR::Entity')) {
                 my $inheritance = UR::Object::Inheritance->create(
+                    type_name => $class->type_name,
                     class_name => $class->class_name,
                     parent_class_name => "UR::Entity",
+                    parent_type_name => "table row",
                     inheritance_priority => 0,
                 );
                 Carp::confess("Failed to generate inheritance link!?") unless $inheritance;
@@ -1011,9 +995,6 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
     } # next table
 
     $self->status_message("Updating class properties...\n");
-
-    $self->status_message($_) foreach @saved_removed_column_messages;
-
     # COLUMN
     my @column_property_translations = (
         ['data_type'    => 'data_type'],
@@ -1080,9 +1061,9 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             if ($property->__changes__) {
                 no warnings;
                 $self->status_message(
-                    sprintf("U %-40s property %-20s for column %s.%s (%s %s)\n",
+                    #sprintf("U %-40s uses table %-40s" . "\n",$class_name,$table_name)
+                    sprintf("U %-40s has updated column %s.%s (%s %s)\n",
                                                             $class_name,
-                                                            $property->property_name,
                                                             $table->table_name, 
                                                             $column_name,
                                                             $column->data_type,
@@ -1126,9 +1107,8 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
 
             no warnings;
             $self->status_message(
-                sprintf("A %-40s property %-16s for column %s.%s (%s %s)\n",
+                sprintf("A %-40s has new column %s.%s (%s %s)\n",
                                                         $class_name,
-                                                        $property_name,
                                                         $table->table_name, 
                                                         $column_name,
                                                         $column->data_type,
@@ -1182,11 +1162,7 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
                 class_name=> $class_name
             );
         
-        #my @expected_pk_cols = map { $class->property_meta_for_name($_->property_name)->column_name } @old_id_properties;
-        my @expected_pk_cols = map { $_->column_name }
-                               grep { defined }
-                               map { $class->property_meta_for_name($_->property_name) }
-                               @old_id_properties;
+        my @expected_pk_cols = map { $class->property_meta_for_name($_->property_name)->column_name } @old_id_properties;
         
         my @pk_cols = $table->primary_key_constraint_column_names;
         
@@ -1243,34 +1219,10 @@ sub  _update_class_metadata_objects_to_match_database_metadata_changes {
             my $name = $property->property_name;
             if ($pk_cols{$name}) {
                 $property->is_optional(0);
-                $property->is_id(1);
             }
         }
     } # next table (looking just for PK constraint changes)
 
-    # Make another pass to make sure if a class has a property called 'id' with a column attached,
-    # then it must be the only ID property of that class
-    my %classes_to_check_id_properties;
-    foreach my $thing ( qw(UR::DataSource::RDBMS::Table UR::DataSource::RDBMS::TableColumn ) ) {
-        foreach my $item ( @{ $dd_changes_by_class{$thing} } ) {
-            my $class_meta = $self->_get_class_meta_for_table_name(data_source => $item->data_source,
-                                                                   table_name => $item->table_name);
-            $classes_to_check_id_properties{$class_meta->class_name} ||= $class_meta;
-        }
-    }
-    foreach my $class_name ( keys %classes_to_check_id_properties ) {
-        my $class_meta = $classes_to_check_id_properties{$class_name};
-        my $property_meta = $class_meta->property_meta_for_name('id');
-        if ($property_meta && $property_meta->column_name && scalar($class_meta->direct_id_property_metas) > 1) {
-            $self->warning_message("Class $class_name cannot have multiple ID properties when one concrete ID property is named 'id'. It will likely not function correctly unless it is renamed");
-        }
-        unless ($property_meta->is_id) {
-            $self->warning_message("Class $class_name has a property named 'id' that is not an ID property.  It will likely not function correctly unless it is renamed");
-        }
-    }
-                                         
-
-
     $self->status_message("Updating class unique constraints...\n");
 
     #$DB::single = 1;
@@ -1644,7 +1596,7 @@ sub _sync_filesystem {
                 my $old_module_path = $class_obj->module_path;
                 my $new_module_path = $old_module_path;
                 $new_module_path =~ s/\/$namespace\//\/$namespace\/\.deleted\//;
-                $status_message_this_update .= " (moving $old_module_path to $new_module_path)";
+                $status_message_this_update .= ' (moving $old_module_path $new_module_path)';
                 rename $old_module_path, $new_module_path;
 
                 UR::Context::Transaction->log_change($class_obj, $class_obj->class_name, $class_obj->id, 'rewrite_module_header', Data::Dumper::Dumper({path => $new_module_path, data => $old_file_data}));
@@ -1664,5 +1616,466 @@ sub _sync_filesystem {
     return 1;
 }
 
+#
+# The following two methods do the real work on a per-table basis.
+# They'll possibly be moved into an action for the table object?
+#
+
+
+sub _update_database_metadata_objects_for_table_changes {
+    my ($self,$data_source,$table_name) = @_;
+
+    my @column_objects;
+    my @all_constraints;
+
+    # this must be on or before the actual data dictionary queries
+    my $revision_time = UR::Time->now();
+
+    # TABLE
+    my $table_sth = $data_source->get_table_details_from_data_dictionary('%', $data_source->owner, $table_name, "TABLE,VIEW");
+    my $table_data = $table_sth->fetchrow_hashref();
+    unless ($table_data && %$table_data) {
+        #Carp::confess("No data for table $table_name in data source $data_source?!");
+        Carp::cluck("No data for table $table_name in data source $data_source?!");
+        return undef;
+    }
+
+    my $data_source_id = $data_source->id;
+    my $table_object = UR::DataSource::RDBMS::Table->get(data_source => $data_source_id,
+                                                         table_name => $table_name);
+    if ($table_object) {
+        # Already exists, update the existing entry
+        # Instead of deleting and recreating the table object (the old way),
+        # modify its attributes in-place.  The name can't change but all the other
+        # stuff might.
+        $table_object->table_type($table_data->{TABLE_TYPE});
+        $table_object->owner($table_data->{TABLE_SCHEM});
+        $table_object->data_source($data_source->class);
+        $table_object->remarks($table_data->{REMARKS});
+        $table_object->last_object_revision($revision_time) if ($table_object->__changes__());
+
+    } else {
+        # Create a brand new one from scratch
+
+        $table_object = UR::DataSource::RDBMS::Table->create(
+            table_name => $table_name,
+            table_type => $table_data->{TABLE_TYPE},
+            owner => $table_data->{TABLE_SCHEM},
+            data_source => $data_source_id,
+            remarks => $table_data->{REMARKS},
+            last_object_revision => $revision_time,
+        );
+        unless ($table_object) {
+            Carp::confess("Failed to get/create table object for $table_name");
+        }
+    }
+
+
+    # COLUMNS
+    # mysql databases seem to require you to actually put in the database name in the first arg
+    my $db_name = ($data_source->can('db_name')) ? $data_source->db_name : '%';
+    my $column_sth = $data_source->get_column_details_from_data_dictionary($db_name, $data_source->owner, $table_name, '%');
+    unless ($column_sth) {
+        $self->error_message("Error getting column data for table $table_name in data source $data_source.");
+        return;
+    }
+    my $all_column_data = $column_sth->fetchall_arrayref({});
+    unless (@$all_column_data) {
+        $self->error_message("No column data for table $table_name in data source $data_source_id");
+        return;
+    }
+    
+    my %columns_to_delete = map {$_->column_name, $_} UR::DataSource::RDBMS::TableColumn->get(table_name => $table_name,
+                                                                                              data_source => $data_source_id);
+    
+    
+    
+    for my $column_data (@$all_column_data) {
+
+        #my $id = $table_name . '.' . $column_data->{COLUMN_NAME}
+        $column_data->{'COLUMN_NAME'} =~ s/"|'//g;  # Postgres puts quotes around things that look like keywords
+        $column_data->{'COLUMN_NAME'} = uc($column_data->{'COLUMN_NAME'});
+        
+        delete $columns_to_delete{$column_data->{'COLUMN_NAME'}};
+        
+        my $column_obj = UR::DataSource::RDBMS::TableColumn->get(table_name => $table_name,
+                                                                 data_source => $data_source_id,
+                                                                 column_name => $column_data->{'COLUMN_NAME'});
+        if ($column_obj) {
+            # Already exists, change the attributes
+            $column_obj->owner($table_object->{owner});
+            $column_obj->data_source($table_object->{data_source});
+            $column_obj->data_type($column_data->{TYPE_NAME});
+            $column_obj->nullable(substr($column_data->{IS_NULLABLE}, 0, 1));
+            $column_obj->data_length($column_data->{COLUMN_SIZE});
+            $column_obj->remarks($column_data->{REMARKS});
+            $column_obj->last_object_revision($revision_time) if ($column_obj->__changes__());
+
+        } else {
+            # It's new, create it from scratch
+            
+            $column_obj = UR::DataSource::RDBMS::TableColumn->create(
+                column_name => $column_data->{COLUMN_NAME},
+                table_name  => $table_object->{table_name},
+                owner       => $table_object->{owner},
+                data_source => $table_object->{data_source},
+        
+                data_type   => $column_data->{TYPE_NAME},
+                nullable    => substr($column_data->{IS_NULLABLE}, 0, 1),
+                data_length => $column_data->{COLUMN_SIZE},
+                remarks     => $column_data->{REMARKS},
+                last_object_revision => $revision_time,
+            );
+        }
+
+        unless ($column_obj) {
+            Carp::confess("Failed to create a column ".$column_data->{'COLUMN_NAME'}." for table $table_name");
+        }
+
+        push @column_objects, $column_obj;
+    }
+    
+    for my $to_delete (values %columns_to_delete) {
+        $self->status_message("Detected column " . $to_delete->column_name . " has gone away.");
+        $to_delete->delete;
+    }
+
+
+    my $bitmap_data = $data_source->get_bitmap_index_details_from_data_dictionary($table_name);
+    for my $index (@$bitmap_data) {
+        #push @{ $embed{bitmap_index_names}{$table_object} }, $index->{'index_name'};
+
+        my $column_object = UR::DataSource::RDBMS::TableColumn->is_loaded(
+            table_name => uc($index->{'table_name'}),
+            data_source => $data_source_id,
+            column_name => uc($index->{'column_name'}),
+        );
+    }
+
+    
+    # Make a note of what FKs exist in the Meta DB involving this table
+    my @fks_in_meta_db = UR::DataSource::RDBMS::FkConstraint->get(data_source => $data_source_id,
+                                                                  table_name => $table_name);
+    push @fks_in_meta_db, UR::DataSource::RDBMS::FkConstraint->get(data_source => $data_source_id,
+                                                                   r_table_name => $table_name);
+    my %fks_in_meta_db_by_fingerprint;
+    foreach my $fk ( @fks_in_meta_db ) {
+        my $fingerprint = $self->_make_foreign_key_fingerprint($fk);
+        $fks_in_meta_db_by_fingerprint{$fingerprint} = $fk;
+    }
+
+    # constraints on this table against columns in other tables
+   
+
+    my $db_owner = $data_source->owner;
+    my $fk_sth = $data_source->get_foreign_key_details_from_data_dictionary('', $db_owner, $table_name, '', '', '');
+
+    my %fk;     # hold the fk constraints that this
+                # invocation of foreign_key_info created
+
+    my @constraints;
+    my %fks_in_real_db;
+    if ($fk_sth) {
+        while (my $data = $fk_sth->fetchrow_hashref()) {
+            #push @$ref_fks, [@$data{qw(FK_NAME FK_TABLE_NAME)}];
+    
+            foreach ( qw( FK_TABLE_NAME UK_TABLE_NAME FK_NAME FK_COLUMN_NAME UK_COLUMN_NAME ) ) {
+                $data->{$_} = uc($data->{$_});
+            }
+
+            my $fk = UR::DataSource::RDBMS::FkConstraint->get(table_name => $data->{'FK_TABLE_NAME'},
+                                                              data_source => $data_source_id,
+                                                              fk_constraint_name => $data->{'FK_NAME'},
+                                                              r_table_name => $data->{'UK_TABLE_NAME'},
+                                                             );
+    
+            unless ($fk) {
+                # Postgres puts quotes around things that look like keywords
+                foreach ( $data->{'FK_TABLE_NAME'}, $data->{'UK_TABLE_NAME'}, $data->{'UK_TABLE_NAME'}, $data->{'UK_COLUMN_NAME'}) {
+                    s/"|'//g;
+                }
+
+                $fk = UR::DataSource::RDBMS::FkConstraint->create(
+                    fk_constraint_name => $data->{'FK_NAME'},
+                    table_name      => $data->{'FK_TABLE_NAME'},
+                    r_table_name    => $data->{'UK_TABLE_NAME'},
+                    owner           => $table_object->{owner},
+                    r_owner         => $table_object->{owner},
+                    data_source     => $table_object->{data_source},
+                    last_object_revision => $revision_time,
+                );
+    
+                $fk{$fk->id} = $fk;
+            }
+    
+            if ($fk{$fk->id}) {
+                my $fkcol = UR::DataSource::RDBMS::FkConstraintColumn->get_or_create(
+                    fk_constraint_name => $data->{'FK_NAME'},
+                    table_name      => $data->{'FK_TABLE_NAME'},
+                    column_name     => $data->{'FK_COLUMN_NAME'},
+                    r_table_name    => $data->{'UK_TABLE_NAME'},
+                    r_column_name   => $data->{'UK_COLUMN_NAME'},
+                    owner           => $table_object->{owner},
+                    data_source     => $table_object->{data_source},
+                );
+                    
+            }
+    
+            my $fingerprint = $self->_make_foreign_key_fingerprint($fk);
+            $fks_in_real_db{$fingerprint} = $fk;
+
+            push @constraints, $fk;
+        }
+    }
+
+    # get foreign_key_info the other way
+    # constraints on other tables against columns in this table
+
+    my $fk_reverse_sth = $data_source->get_foreign_key_details_from_data_dictionary('', '', '', '', $db_owner, $table_name);
+
+    %fk = ();   # resetting this prevents data_source referencing
+                # tables from fouling up their fk objects
+
+
+    if ($fk_reverse_sth) {
+        while (my $data = $fk_reverse_sth->fetchrow_hashref()) {
+
+            foreach ( qw( FK_TABLE_NAME UK_TABLE_NAME FK_NAME FK_COLUMN_NAME UK_COLUMN_NAME ) ) {
+                $data->{$_} = uc($data->{$_});
+            }
+
+            my $fk = UR::DataSource::RDBMS::FkConstraint->get(fk_constraint_name => $data->{'FK_NAME'},
+                                                              table_name => $data->{'FK_TABLE_NAME'},
+                                                              r_table_name => $data->{'UK_TABLE_NAME'},
+                                                              data_source => $table_object->{'data_source'},
+                                                            );
+            unless ($fk) {
+                # Postgres puts quotes around things that look like keywords
+                foreach ( $data->{'FK_TABLE_NAME'}, $data->{'UK_TABLE_NAME'}, $data->{'UK_TABLE_NAME'}, $data->{'UK_COLUMN_NAME'}) {
+                    s/"|'//g;
+                }
+
+                $fk = UR::DataSource::RDBMS::FkConstraint->create(
+                    fk_constraint_name => $data->{'FK_NAME'},
+                    table_name      => $data->{'FK_TABLE_NAME'},
+                    r_table_name    => $data->{'UK_TABLE_NAME'},
+                    owner           => $table_object->{owner},
+                    r_owner         => $table_object->{owner},
+                    data_source     => $table_object->{data_source},
+                    last_object_revision => $revision_time,
+                );
+                unless ($fk) {
+                    #$DB::single=1;
+                    1;
+                }
+                $fk{$fk->fk_constraint_name} = $fk;
+            }
+    
+            if ($fk{$fk->fk_constraint_name}) {
+                 UR::DataSource::RDBMS::FkConstraintColumn->get_or_create(
+                    fk_constraint_name => $data->{'FK_NAME'},
+                    table_name      => $data->{'FK_TABLE_NAME'},
+                    column_name     => $data->{'FK_COLUMN_NAME'},
+                    r_table_name    => $data->{'UK_TABLE_NAME'},
+                    r_column_name   => $data->{'UK_COLUMN_NAME'},
+                    owner           => $table_object->{owner},
+                    data_source     => $table_object->{data_source},
+                 );
+            }
+    
+                
+            my $fingerprint = $self->_make_foreign_key_fingerprint($fk);
+            $fks_in_real_db{$fingerprint} = $fk;
+
+            push @constraints, $fk;
+        }
+    }
+
+    # Find FKs still in the Meta db that don't exist in the real database anymore
+    foreach my $fingerprint ( keys %fks_in_meta_db_by_fingerprint ) {
+        unless ($fks_in_real_db{$fingerprint}) {
+            my $fk = $fks_in_meta_db_by_fingerprint{$fingerprint};
+            my @fk_cols = $fk->get_related_column_objects();
+            $_->delete foreach @fk_cols;
+            $fk->delete;
+        }
+    }
+
+    # get primary_key_info
+
+    my $pk_sth = $data_source->get_primary_key_details_from_data_dictionary(undef, $db_owner, $table_name);
+
+    if ($pk_sth) {
+		my @new_pk;
+        while (my $data = $pk_sth->fetchrow_hashref()) {
+            $data->{'COLUMN_NAME'} =~ s/"|'//g;  # Postgres puts quotes around things that look like keywords
+            my $pk = UR::DataSource::RDBMS::PkConstraintColumn->get(
+                            table_name => $table_name,
+                            data_source => $data_source_id,
+                            column_name => $data->{'COLUMN_NAME'},
+                          );
+            if ($pk) {
+				# Since the rank/order is pretty much all that might change, we
+				# just delete and re-create these.
+				# It's a no-op at save time if there are no changes.
+            	$pk->delete;
+            }
+			
+			push @new_pk, [
+				table_name => $table_name,
+				data_source => $data_source_id,
+				owner => $data_source->owner,
+				column_name => $data->{'COLUMN_NAME'},
+				rank => $data->{'KEY_SEQ'} || $data->{'ORDINAL_POSITION'},
+			];
+			#        $table_object->{primary_key_constraint_name} = $data->{PK_NAME};
+			#        $embed{primary_key_constraint_column_names} ||= {};
+			#        $embed{primary_key_constraint_column_names}{$table_object} ||= [];
+			#        push @{ $embed{primary_key_constraint_column_names}{$table_object} }, $data->{COLUMN_NAME};
+        }
+		
+		for my $data (@new_pk) {
+        	my $pk = UR::DataSource::RDBMS::PkConstraintColumn->create(@$data);
+			unless ($pk) {
+				$self->error_message("Failed to create primary key @$data");
+				return;
+			}
+		}			
+    }
+
+    ## Get the unique constraints
+    ## Unfortunately, there appears to be no DBI catalog
+    ## method which will find these.  So we have to use
+    ## some custom SQL
+    #
+    # The SQL that used to live here was moved to the UR::DataSource::Oracle
+    # and each other DataSource class needs its own implementation
+
+    # The above was moved into each data source's class
+    if (my $uc = $data_source->get_unique_index_details_from_data_dictionary($table_name)) {
+        my %uc = %$uc;
+
+        # check for redundant unique constraints
+        # there may be both an index and a constraint
+
+        for my $uc_name_1 ( keys %uc ) {
+
+            my $uc_columns_1 = $uc{$uc_name_1}
+                or next;
+            my $uc_columns_1_serial = join ',', sort @$uc_columns_1;
+
+            for my $uc_name_2 ( keys %uc ) {
+                next if ( $uc_name_2 eq $uc_name_1 );
+                my $uc_columns_2 = $uc{$uc_name_2}
+                    or next;
+                my $uc_columns_2_serial = join ',', sort @$uc_columns_2;
+
+                if ( $uc_columns_2_serial eq $uc_columns_1_serial ) {
+                    delete $uc{$uc_name_1};
+                }
+            }
+        }
+
+        # compare primary key constraints to unique constraints
+        my $pk_columns_serial = join(',', sort map { $_->column_name }
+                                            UR::DataSource::RDBMS::PkConstraintColumn->get(data_source => $data_source_id,
+                                                                                           table_name => $table_name,
+                                                                                           owner => $data_source->owner,
+                                                                                         ));
+        for my $uc_name ( keys %uc ) {
+
+            # see if primary key constraint has the same name as
+            # any unique constraints
+            # FIXME - disabling this for now, the Meta DB dosen't track PK constraint names
+            # Isn't it just as goot to check the involved columns?
+            #if ( $table_object->primary_key_constraint_name eq $uc_name ) {
+            #    delete $uc{$uc_name};
+            #    next;
+            #}
+
+            # see if any unique constraints cover the exact same column(s) as
+            # the primary key column(s)
+            my $uc_columns_serial = join ',',
+                sort @{ $uc{$uc_name} };
+
+            if ( $pk_columns_serial eq $uc_columns_serial ) {
+                delete $uc{$uc_name};
+            }
+        }
+
+        # Create new UniqueConstraintColumn objects for the columns that don't exist, and delete the
+        # objects if they don't apply anymore
+        foreach my $uc_name ( keys %uc ) {
+            my %constraint_objs = map { $_->column_name => $_ } UR::DataSource::RDBMS::UniqueConstraintColumn->get(
+                                                                            data_source => $data_source_id,
+                                                                            table_name => $table_name,
+                                                                            owner => $data_source->owner || '',
+                                                                            constraint_name => $uc_name,
+                                                                          );
+    
+            foreach my $col_name ( @{$uc{$uc_name}} ) {
+                if ($constraint_objs{$col_name} ) {
+                    delete $constraint_objs{$col_name};
+                } else {
+                    my $uc = UR::DataSource::RDBMS::UniqueConstraintColumn->create(
+                                                   data_source => $data_source_id,
+                                                   table_name => $table_name,
+                                                   owner => $data_source->owner,
+                                                   constraint_name => $uc_name,
+                                                   column_name => $col_name,
+                                              );
+                     1;
+                }
+            } 
+            foreach my $obj ( values %constraint_objs ) {
+                $obj->delete();
+            }
+        }
+    }
+
+    # Now that all columns know their foreign key constraints,
+    # have the column objects resolve the various names
+    # associated with the column.
+
+    #for my $col (@column_objects) { $col->resolve_names }
+
+    # Determine the ER type.
+    # We have 'validation item', 'entity', and 'bridge'
+
+    my $column_count = scalar($table_object->column_names) || 0;
+    my $pk_column_count = scalar($table_object->primary_key_constraint_column_names) || 0;
+    my $constraint_count = scalar($table_object->fk_constraint_names) || 0;
+
+    if ($column_count == 1 and $pk_column_count == 1)
+    {
+        $table_object->er_type('validation item');
+    }
+    else
+    {
+        if ($constraint_count == $column_count)
+        {
+            $table_object->er_type('bridge');
+        }
+        else
+        {
+            $table_object->er_type('entity');
+        }
+    }
+
+    return $table_object;
+}
+
+sub _make_foreign_key_fingerprint {
+    my($self,$fk) = @_;
+
+    my @fk_cols = sort {$a->column_name cmp $b->column_name} $fk->get_related_column_objects();
+    my $fingerprint = join(':', $fk->table_name,
+                                $fk->r_table_name,
+                                map { $_->column_name, $_->r_column_name } @fk_cols);
+    return $fingerprint;
+}
+
+
+
 1;
 
