@@ -6,26 +6,18 @@ package UR::Namespace::Command::Update::SchemaDiagram;
 use strict;
 use warnings;
 use UR;
-our $VERSION = "0.46"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
-    is => 'UR::Namespace::Command::Base',
+    is => 'UR::Namespace::Command',
     has => [ 
         data_source => {type => 'String', doc => 'Which datasource to use', is_optional => 1},
         depth => { type => 'Integer', doc => 'Max distance of related tables to include.  Default is 1.  0 means show only the named tables, -1 means to include everything', is_optional => 1},
         file => { type => 'String', doc => 'Pathname of the Umlet (.uxf) file' },
         show_columns => { type => 'Boolean', is_optional => 1, default => 1, doc => 'Include column names in the diagram' },
-        initial_name => {
-            is_many => 1,
-            is_optional => 1,
-            shell_args_position => 1
-        }
     ],
 );
 
-sub sub_command_sort_position { 3 };
-
 
 sub help_brief {
     "Update an Umlet diagram based on the current schema"
@@ -51,10 +43,9 @@ use constant MAX_X_AUTO_POSITION => 1000;
 # most of the code
 sub execute {
     my $self = shift;
-
     my $params = shift;
     
-#$DB::single = 1;
+$DB::single=1;
     my $namespace = $self->namespace_name;
     eval "use $namespace";
     if ($@) {
@@ -74,7 +65,7 @@ sub execute {
         @initial_name_list = map { $_->table_name}
                                  UR::DataSource::RDBMS::Table->get(namespace => $namespace);
     } else {
-        @initial_name_list = $self->initial_name;
+        @initial_name_list = @{$params->{' '}};
     }
 
     my $diagram;
@@ -100,7 +91,7 @@ sub execute {
     #    push @involved_tables, UR::DataSource::RDBMS::Table->get(namespace => $namespace, table_name => $table_name);
     #}
 
-#$DB::single = 1;
+$DB::single=1;
     push @involved_tables ,$self->_get_related_items( names => \@initial_name_list,
                                                       depth => $params->{'depth'},
                                                       namespace => $namespace,
