@@ -4,7 +4,6 @@ use strict;
 use warnings;
 
 use UR;
-our $VERSION = "0.46"; # UR $VERSION;
 use Data::Dumper;
 
 class UR::Object::Command::FetchAndDo {
@@ -43,43 +42,12 @@ sub _filter_doc {
     my $doc = <<EOS;
 Filtering:
 ----------
- Create filter equations by combining filterable properties with operators and
-     values.
+ Create filter equations by combining filterable properties with operators and values.
  Combine and separate these 'equations' by commas.  
- Use single quotes (') to contain values with spaces: name='genome institute'
+ Use single quotes (') to contain values with spaces: name='genome center'
  Use percent signs (%) as wild cards in like (~).
- Use backslash or single quotes to escape characters which have special meaning
-     to the shell such as < > and &
-
-Operators:
-----------
- =  (exactly equal to)
- ~  (like the value)
- :  (in the list of several values, slash "/" separated)
-    (or between two values, dash "-" separated)
- >  (greater than)
- >= (greater than or equal to)
- <  (less than)
- <= (less than or equal to)
-
-Examples:
----------
-EOS
-    if (my $help_synopsis = $class->help_synopsis) {
-        $doc .= " $help_synopsis\n";
-    } else {
-        $doc .= <<EOS
- lister-command --filter name=Bob --show id,name,address
- lister-command --filter name='something with space',employees\>200,job~%manager
- lister-command --filter cost:20000-90000
- lister-command --filter answer:yes/maybe
-EOS
-    }
-
-    $doc .= <<EOS;
 
 Filterable Properties: 
-----------------------
 EOS
 
     # Try to get the subject class name
@@ -91,33 +59,14 @@ EOS
 
     if ( $self->subject_class_name ) {
         if ( my @properties = $self->_subject_class_filterable_properties ) {
-            my $longest_name = 0;
-            foreach my $property ( @properties ) {
-                my $name_len = length($property->property_name);
-                $longest_name = $name_len if ($name_len > $longest_name);
-            }
-
             for my $property ( @properties ) {
-                my $property_doc = $property->doc;
-                unless ($property_doc) {
-                    eval {
-                        foreach my $ancestor_class_meta ( $property->class_meta->ancestry_class_metas ) {
-                            my $ancestor_property_meta = $ancestor_class_meta->property_meta_for_name($property->property_name);
-                            if ($ancestor_property_meta and $ancestor_property_meta->doc) {
-                                $property_doc = $ancestor_property_meta->doc;
-                                last;
-                            }
-                        }
-                    };
-                }
-                $property_doc ||= ' (undocumented)';
-                $property_doc =~ s/\n//gs;   # Get rid of embeded newlines
-
-                my $data_type = $property->data_type || '';
-                $data_type = ucfirst(lc $data_type);
-
-                $doc .= sprintf(" %${longest_name}s  ($data_type): $property_doc\n",
-                                $property->property_name);
+                $doc .= sprintf(" %s\n", $property->property_name);
+                next; # TODO doc??
+                $doc .= sprintf(
+                    " %s: %s\n",
+                    $property->property_name,
+                    ( $property->description || 'no doc' ),
+                );
             }
         }
         else {
@@ -125,10 +74,24 @@ EOS
         }
     }
     else {
-        $doc .= " Can't determine the list of filterable properties without a subject_class_name";
+        $doc .= " Need subject class name to get properties.\n"
     }
 
-    return $doc;
+    $doc .= <<EOS;
+
+Operators:
+ =  (exactly equal to)
+ ~  (like the value)
+ >  (greater than)
+ >= (greater than or equal to)
+ <  (less than)
+ <= (less than or equal to)
+
+Examples:
+ name='genome center'
+ employees>200
+ name~genome%,employees>200
+EOS
 }
 
 ########################################################################
@@ -160,7 +123,7 @@ sub _validate_subject_class {
     )
         and return if $subject_class_name =~ /^UR::/;
 
-    UR::Object::Type->use_module_with_namespace_constraints($subject_class_name);
+    eval "use $subject_class_name;"; # dont check for errors
     
     my $subject_class = $self->subject_class;
     $self->error_message(
@@ -188,15 +151,11 @@ sub _subject_class_filterable_properties {
     $self->_validate_subject_class
         or return;
 
-    my %props = map { $_->property_name => $_ }
-                    $self->subject_class->property_metas;
-
-    return map { $_->[1] }                   # These maps are to get around a bug in perl 5.8
-           sort { $a->[0] cmp $b->[0] }      # sort involving methdo calls inside the sort sub that
-           map { [ $_->property_name, $_ ] } # might do sorts of their own
-           grep { substr($_->property_name, 0, 1) ne '_' }  # Skip 'private' properties starting with '_'
-           grep { ! $_->data_type or index($_->data_type, '::') == -1 }  # Can't filter object-type properties from a lister, right?
-           values %props;
+    return sort { 
+        $a->property_name cmp $b->property_name
+    } grep {
+        $_->column_name ne ''
+    } $self->subject_class->all_property_metas;
 }
 
 sub _hint_string {
@@ -224,7 +183,7 @@ sub _fetch
     $self->error_message( sprintf('Unrecognized field(s): %s', join(', ', keys %extra)) )
         and return if %extra;
     
-    if (my $i = $self->subject_class_name->create_iterator($bool_expr)) {
+    if (my $i = $self->subject_class_name->create_iterator(where => $bool_expr)) {
         return $i;
     }
     else {
@@ -323,5 +282,5 @@ Overwrite the help_brief, help_synopsis and help_detail methods to provide speci
 
 =cut
 
-#$HeadURL: svn+ssh://svn/srv/svn/gscpan/distro/ur-bundle/trunk/lib/UR/Object/Command/FetchAndDo.pm $
+#$HeadURL: svn+ssh://svn/srv/svn/gscpan/distro/ur-bundle/releases/UR-0.5/lib/UR/Object/Command/FetchAndDo.pm $
 #$Id: FetchAndDo.pm 47408 2009-06-01 03:53:45Z ssmith $#
